"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

from decimal import Decimal

import pytest

import typepy

from ._common import EXCEPTION_RESULT, convert_wrapper


nan = float("nan")
inf = float("inf")


class Test_RealNumber:
    @pytest.mark.parametrize(
        ["method", "strict_level", "value", "expected"],
        [
            ["convert", 0, 1, Decimal("1")],
            ["convert", 0, 1.0, Decimal("1.0")],
            ["convert", 0, 1.1, Decimal("1.1")],
            ["convert", 0, "0", Decimal("0")],
            ["convert", 0, "1.0", Decimal("1.0")],
            ["convert", 0, "1.1", Decimal("1.1")],
            ["convert", 0, True, EXCEPTION_RESULT],
            ["convert", 0, None, EXCEPTION_RESULT],
            ["convert", 0, inf, EXCEPTION_RESULT],
            ["convert", 0, nan, EXCEPTION_RESULT],
            ["convert", 0, "test", EXCEPTION_RESULT],
            ["convert", 0, "", EXCEPTION_RESULT],
            ["convert", 1, 1, EXCEPTION_RESULT],
            ["convert", 1, 1.0, EXCEPTION_RESULT],
            ["convert", 1, 1.1, Decimal("1.1")],
            ["convert", 1, "0", EXCEPTION_RESULT],
            ["convert", 1, "1.0", EXCEPTION_RESULT],
            ["convert", 1, "1.1", Decimal("1.1")],
            ["convert", 1, True, EXCEPTION_RESULT],
            ["convert", 1, None, EXCEPTION_RESULT],
            ["convert", 1, inf, EXCEPTION_RESULT],
            ["convert", 1, nan, EXCEPTION_RESULT],
            ["convert", 1, "test", EXCEPTION_RESULT],
            ["convert", 1, "", EXCEPTION_RESULT],
            ["convert", 2, 1, EXCEPTION_RESULT],
            ["convert", 2, 1.0, EXCEPTION_RESULT],
            ["convert", 2, 1.1, Decimal("1.1")],
            ["convert", 2, "0", EXCEPTION_RESULT],
            ["convert", 2, "1.0", EXCEPTION_RESULT],
            ["convert", 2, "1.1", EXCEPTION_RESULT],
            ["convert", 2, True, EXCEPTION_RESULT],
            ["convert", 2, None, EXCEPTION_RESULT],
            ["convert", 2, inf, EXCEPTION_RESULT],
            ["convert", 2, nan, EXCEPTION_RESULT],
            ["convert", 2, "test", EXCEPTION_RESULT],
            ["convert", 2, "", EXCEPTION_RESULT],
            ["try_convert", 0, 1, Decimal("1")],
            ["try_convert", 0, 1.0, Decimal("1.0")],
            ["try_convert", 0, 1.1, Decimal("1.1")],
            ["try_convert", 0, "0", Decimal("0")],
            ["try_convert", 0, "1.0", Decimal("1.0")],
            ["try_convert", 0, "1.1", Decimal("1.1")],
            ["try_convert", 0, True, None],
            ["try_convert", 0, None, None],
            ["try_convert", 0, inf, None],
            ["try_convert", 0, nan, None],
            ["try_convert", 0, "test", None],
            ["try_convert", 0, "", None],
            ["try_convert", 1, 1, None],
            ["try_convert", 1, 1.0, None],
            ["try_convert", 1, 1.1, Decimal("1.1")],
            ["try_convert", 1, "0", None],
            ["try_convert", 1, "1.0", None],
            ["try_convert", 1, "1.1", Decimal("1.1")],
            ["try_convert", 1, True, None],
            ["try_convert", 1, None, None],
            ["try_convert", 1, inf, None],
            ["try_convert", 1, nan, None],
            ["try_convert", 1, "test", None],
            ["try_convert", 1, "", None],
            ["try_convert", 2, 1, None],
            ["try_convert", 2, 1.0, None],
            ["try_convert", 2, 1.1, Decimal("1.1")],
            ["try_convert", 2, "0", None],
            ["try_convert", 2, "1.0", None],
            ["try_convert", 2, "1.1", None],
            ["try_convert", 2, True, None],
            ["try_convert", 2, None, None],
            ["try_convert", 2, inf, None],
            ["try_convert", 2, nan, None],
            ["try_convert", 2, "test", None],
            ["try_convert", 2, "", None],
            ["force_convert", 0, 1, Decimal("1")],
            ["force_convert", 0, 1.0, Decimal("1.0")],
            ["force_convert", 0, 1.1, Decimal("1.1")],
            ["force_convert", 0, "0", Decimal("0")],
            ["force_convert", 0, "1.0", Decimal("1.0")],
            ["force_convert", 0, "1.1", Decimal("1.1")],
            ["force_convert", 0, True, Decimal("1")],
            ["force_convert", 0, None, EXCEPTION_RESULT],
            ["force_convert", 0, inf, Decimal("inf")],
            ["force_convert", 0, nan, Decimal("nan")],
            ["force_convert", 0, "test", EXCEPTION_RESULT],
            ["force_convert", 0, "", EXCEPTION_RESULT],
            ["force_convert", 1, 1, Decimal("1")],
            ["force_convert", 1, 1.0, Decimal("1.0")],
            ["force_convert", 1, 1.1, Decimal("1.1")],
            ["force_convert", 1, "0", Decimal("0")],
            ["force_convert", 1, "1.0", Decimal("1.0")],
            ["force_convert", 1, "1.1", Decimal("1.1")],
            ["force_convert", 1, True, Decimal("1")],
            ["force_convert", 1, None, EXCEPTION_RESULT],
            ["force_convert", 1, inf, Decimal("inf")],
            ["force_convert", 1, nan, Decimal("nan")],
            ["force_convert", 1, "test", EXCEPTION_RESULT],
            ["force_convert", 1, "", EXCEPTION_RESULT],
            ["force_convert", 2, 1, Decimal("1")],
            ["force_convert", 2, 1.0, Decimal("1.0")],
            ["force_convert", 2, 1.1, Decimal("1.1")],
            ["force_convert", 2, "0", Decimal("0")],
            ["force_convert", 2, "1.0", Decimal("1.0")],
            ["force_convert", 2, "1.1", Decimal("1.1")],
            ["force_convert", 2, True, Decimal("1")],
            ["force_convert", 2, None, EXCEPTION_RESULT],
            ["force_convert", 2, inf, Decimal("inf")],
            ["force_convert", 2, nan, Decimal("nan")],
            ["force_convert", 2, "test", EXCEPTION_RESULT],
            ["force_convert", 2, "", EXCEPTION_RESULT],
        ],
    )
    def test_normal(self, method, strict_level, value, expected):
        from typepy import Nan, StrictLevel

        actual = convert_wrapper(typepy.RealNumber(value, strict_level), method)
        if Nan(expected, strict_level=StrictLevel.MIN).is_type():
            assert Nan(actual, strict_level=StrictLevel.MIN).is_type()
        else:
            assert actual == expected



"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

import itertools

import pytest
from tcolorpy import tcolor

from typepy import Bool, StrictLevel, Typecode


class_under_test = Bool
nan = float("nan")
inf = float("inf")


class Test_Bool_is_type:
    @pytest.mark.parametrize(
        ["value", "strict_level", "expected"],
        list(itertools.product([True, False], [StrictLevel.MIN, StrictLevel.MAX], [True]))
        + list(
            itertools.product([0, 1, "True", "False", "true", "false"], [StrictLevel.MIN], [True])
        )
        + list(
            itertools.product(
                [0, 1, 0.1, "True", "False", "true", "false"], [StrictLevel.MAX], [False]
            )
        ),
    )
    def test_normal(self, value, strict_level, expected):
        type_checker = class_under_test(value, strict_level)

        assert type_checker.is_type() == expected
        assert type_checker.typecode == Typecode.BOOL

    @pytest.mark.parametrize(
        ["value", "strip_ansi_escape", "expected"],
        [[tcolor("True", "red"), False, False], [tcolor("True", "red"), True, True]],
    )
    def test_normal_ansi(self, value, strip_ansi_escape, expected):
        type_checker = class_under_test(value, StrictLevel.MIN, strip_ansi_escape=strip_ansi_escape)

        assert type_checker.is_type() == expected
        assert type_checker.typecode == Typecode.BOOL



"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

import pytest

import typepy

from ._common import convert_wrapper


class Test_Bool:
    @pytest.mark.parametrize(
        ["method", "strict_level", "value", "expected"],
        [
            ["convert", 0, True, True],
            ["convert", 0, False, False],
            ["convert", 0, "true", True],
            ["convert", 0, "FALSE", False],
            ["convert", 0, 1, True],
            ["convert", 0, 1.1, "E"],
            ["convert", 0, None, "E"],
            ["convert", 1, True, True],
            ["convert", 1, "true", True],
            ["convert", 1, "FALSE", False],
            ["convert", 1, 1, "E"],
            ["convert", 1, 1.1, "E"],
            ["convert", 1, None, "E"],
            ["convert", 2, True, True],
            ["convert", 2, "true", "E"],
            ["convert", 2, "FALSE", "E"],
            ["convert", 2, 1, "E"],
            ["convert", 2, 1.1, "E"],
            ["convert", 2, None, "E"],
            ["try_convert", 0, True, True],
            ["try_convert", 0, "true", True],
            ["try_convert", 0, "FALSE", False],
            ["try_convert", 0, 1, True],
            ["try_convert", 0, 1.1, None],
            ["try_convert", 0, None, None],
            ["try_convert", 1, True, True],
            ["try_convert", 1, "true", True],
            ["try_convert", 1, "FALSE", False],
            ["try_convert", 1, 1, None],
            ["try_convert", 1, 1.1, None],
            ["try_convert", 1, None, None],
            ["try_convert", 2, True, True],
            ["try_convert", 2, "true", None],
            ["try_convert", 2, "FALSE", None],
            ["try_convert", 2, 1, None],
            ["try_convert", 2, 1.1, None],
            ["try_convert", 2, None, None],
            ["force_convert", 0, True, True],
            ["force_convert", 0, "true", True],
            ["force_convert", 0, "FALSE", False],
            ["force_convert", 0, 1, True],
            ["force_convert", 0, 1.1, "E"],
            ["force_convert", 0, None, "E"],
            ["force_convert", 1, True, True],
            ["force_convert", 1, "true", True],
            ["force_convert", 1, "FALSE", False],
            ["force_convert", 1, 1, True],
            ["force_convert", 1, 1.1, "E"],
            ["force_convert", 1, None, "E"],
            ["force_convert", 2, True, True],
            ["force_convert", 2, "true", True],
            ["force_convert", 2, "FALSE", False],
            ["force_convert", 2, 1, True],
            ["force_convert", 2, 1.1, "E"],
            ["force_convert", 2, None, "E"],
        ],
    )
    def test_normal(self, method, strict_level, value, expected):
        assert convert_wrapper(typepy.Bool(value, strict_level), method) == expected



"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

from .._common import strip_ansi_escape
from .._const import DefaultValue, ParamKey
from ..error import TypeConversionError
from ._interface import AbstractValueConverter


class BoolConverter(AbstractValueConverter):
    def force_convert(self):
        if isinstance(self._value, int):
            return bool(self._value)

        try:
            return self.__strict_strtobool(self._value)
        except ValueError:
            pass

        if self._params.get(ParamKey.STRIP_ANSI_ESCAPE, DefaultValue.STRIP_ANSI_ESCAPE):
            try:
                return self.__strict_strtobool(strip_ansi_escape(self._value))
            except (TypeError, ValueError):
                pass

        raise TypeConversionError(
            "failed to force_convert to bool: type={}".format(type(self._value))
        )

    @staticmethod
    def __strict_strtobool(value):
        if isinstance(value, bool):
            return value

        try:
            lower_text = value.lower()
        except AttributeError:
            raise ValueError("invalid value '{}'".format(str(value)))

        if lower_text in ["true"]:
            return True
        elif lower_text in ["false"]:
            return False

        raise ValueError("invalid value '{}'".format(str(value)))



"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

import itertools

import pytest

from typepy import List, StrictLevel, Typecode


nan = float("nan")
inf = float("inf")


class Test_List_is_type:
    @pytest.mark.parametrize(
        ["value", "strict_level", "expected"],
        [
            [("a", 1), StrictLevel.MIN, True],
            [("a", 1), StrictLevel.MAX, False],
            [{"a": 1, "b": 2}, StrictLevel.MIN, True],
            [{"a": 1, "b": 2}, StrictLevel.MAX, False],
        ]
        + list(itertools.product([[], ["a", 1]], [StrictLevel.MIN, StrictLevel.MAX], [True]))
        + list(
            itertools.product(
                [(), {}, "", "abc", 1, nan, True, None], [StrictLevel.MIN, StrictLevel.MAX], [False]
            )
        ),
    )
    def test_normal(self, value, strict_level, expected):
        type_object = List(value, strict_level)

        assert type_object.is_type() == expected
        assert type_object.typecode == Typecode.LIST



"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

from ._bool import BoolTypeChecker
from ._bytes import BytesTypeChecker
from ._datetime import DateTimeTypeChecker
from ._dictionary import DictionaryTypeChecker
from ._infinity import InfinityTypeChecker
from ._integer import IntegerTypeChecker
from ._interface import TypeCheckerInterface
from ._ipaddress import IpAddressTypeChecker
from ._list import ListTypeChecker
from ._nan import NanTypeChecker
from ._none import NoneTypeChecker
from ._realnumber import RealNumberTypeChecker
from ._string import NullStringTypeChecker, StringTypeChecker



"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

from ._checker import CheckerFactory, TypeCheckerBase, TypeCheckerDelegator


class BytesTypeCheckerStrictLevel0(TypeCheckerBase):
    def is_instance(self):
        return isinstance(self._value, bytes)

    def is_valid_after_convert(self, converted_value):
        return isinstance(converted_value, bytes)


_factory = CheckerFactory(checker_mapping={0: BytesTypeCheckerStrictLevel0})


class BytesTypeChecker(TypeCheckerDelegator):
    def __init__(self, value, strict_level):
        super().__init__(value=value, checker_factory=_factory, strict_level=strict_level)



"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

import itertools
import sys
from decimal import Decimal

import pytest

from typepy import Nan, StrictLevel, Typecode


nan = float("nan")
inf = float("inf")


class Test_Nan_is_type:
    @pytest.mark.parametrize(
        ["value", "strict_level", "expected"],
        list(
            itertools.product(
                [0.0, sys.maxsize, "0", inf, None], [StrictLevel.MIN, StrictLevel.MAX], [False]
            )
        )
        + list(itertools.product([nan, Decimal("NaN")], [StrictLevel.MIN, StrictLevel.MAX], [True]))
        + [
            ["nan", StrictLevel.MIN, True],
            ["nan", StrictLevel.MAX, False],
            ["-Nan", StrictLevel.MIN, True],
            ["-Nan", StrictLevel.MAX, False],
            ["NAN", StrictLevel.MIN, True],
            ["NAN", StrictLevel.MAX, False],
        ],
    )
    def test_normal(self, value, strict_level, expected):
        type_checker = Nan(value, strict_level)

        assert type_checker.is_type() == expected
        assert type_checker.typecode == Typecode.NAN



"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

import abc
from typing import Any, Optional

from .._typecode import Typecode
from ..checker._interface import TypeCheckerInterface
from ..converter import ValueConverterInterface
from ..error import TypeConversionError


class AbstractType(TypeCheckerInterface, ValueConverterInterface):
    __slots__ = (
        "_data",
        "_strict_level",
        "_params",
        "__checker",
        "__converter",
        "__is_type_result",
    )

    @abc.abstractproperty
    def typecode(self) -> Typecode:  # pragma: no cover
        pass

    @abc.abstractmethod
    def _create_type_checker(self):  # pragma: no cover
        pass

    @abc.abstractmethod
    def _create_type_converter(self):  # pragma: no cover
        pass

    @property
    def typename(self) -> str:
        return self.typecode.name

    def __init__(self, value: Any, strict_level: int, **kwargs) -> None:
        self._data = value
        self._strict_level = strict_level
        self._params = kwargs

        self.__checker = self._create_type_checker()
        self.__converter = self._create_type_converter()

        self.__is_type_result = None  # type: Optional[bool]

    def __repr__(self) -> str:
        return ", ".join(
            [
                "value={}".format(self._data),
                "typename={}".format(self.typename),
                "strict_level={}".format(self._strict_level),
                "is_type={}".format(self.is_type()),
                "try_convert={}".format(self.try_convert()),
            ]
        )

    def is_type(self) -> bool:
        """
        :return:
        :rtype: bool
        """

        if self.__is_type_result is not None:
            return self.__is_type_result

        self.__is_type_result = self.__is_type()

        return self.__is_type_result

    def __is_type(self) -> bool:
        if self.__checker.is_type():
            return True

        if self.__checker.is_exclude_instance():
            return False

        try:
            self._converted_value = self.__converter.force_convert()
        except TypeConversionError:
            return False

        if not self.__checker.is_valid_after_convert(self._converted_value):
            return False

        return True

    def validate(self, error_message: Optional[str] = None) -> None:
        """
        :raises TypeError:
            If the value is not matched the type that the class represented.
        """

        if self.is_type():
            return

        if not error_message:
            error_message = "invalid value type"

        raise TypeError(
            "{}: expected={}, actual={}".format(error_message, self.typename, type(self._data))
        )

    def convert(self):
        """
        :return: Converted value.
        :raises typepy.TypeConversionError:
            If the value cannot convert.
        """

        if self.is_type():
            return self.force_convert()

        raise TypeConversionError(
            "failed to convert {} from {} to {}".format(
                self._data, type(self._data).__name__, self.typename
            )
        )

    def force_convert(self):
        """
        :return: Converted value.
        :raises typepy.TypeConversionError:
            If the value cannot convert.
        """

        return self.__converter.force_convert()

    def try_convert(self):
        """
        :return: Converted value. |None| if failed to convert.
        """

        try:
            return self.convert()
        except TypeConversionError:
            return None



"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

from decimal import Decimal, InvalidOperation

from .._common import strip_ansi_escape
from .._const import DefaultValue, ParamKey
from ..error import TypeConversionError
from ._interface import AbstractValueConverter


class IntegerConverter(AbstractValueConverter):
    def force_convert(self):
        try:
            return int(Decimal(self._value))
        except (TypeError, OverflowError, ValueError, InvalidOperation):
            pass

        if self._params.get(ParamKey.STRIP_ANSI_ESCAPE, DefaultValue.STRIP_ANSI_ESCAPE):
            try:
                return int(Decimal(strip_ansi_escape(self._value)))
            except (TypeError, OverflowError, ValueError, InvalidOperation):
                pass

        raise TypeConversionError(
            "failed to force_convert to int: type={}".format(type(self._value))
        )



"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

from ._bool import BoolConverter
from ._bytes import BytesConverter
from ._datetime import DateTimeConverter
from ._dictionary import DictionaryConverter
from ._integer import IntegerConverter
from ._interface import ValueConverterInterface
from ._ipaddress import IpAddressConverter
from ._list import ListConverter
from ._nop import NopConverter
from ._realnumber import FloatConverter
from ._string import NullStringConverter, StringConverter



"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

import itertools
from collections import OrderedDict

import pytest

from typepy import Dictionary, StrictLevel, Typecode


nan = float("nan")
inf = float("inf")


class Test_Dictionary_is_type:
    @pytest.mark.parametrize(
        ["value", "strict_level", "expected"],
        [
            [[["a", 1]], StrictLevel.MIN, True],
            [[["a", 1]], StrictLevel.MAX, False],
            [(("a", 1),), StrictLevel.MIN, True],
            [(("a", 1),), StrictLevel.MAX, False],
        ]
        + list(
            itertools.product(
                [{}, {"a": 1, "b": 2}, OrderedDict(a=1, b=1)],
                [StrictLevel.MIN, StrictLevel.MAX],
                [True],
            )
        )
        + [
            ['{"foo": 10}', StrictLevel.MIN, True],
            ['{"foo": 10}', StrictLevel.MAX, False],
        ]
        + list(
            itertools.product(
                [1, "a", "あ".encode("utf8"), nan, True, None],
                [StrictLevel.MIN, StrictLevel.MAX],
                [False],
            )
        ),
    )
    def test_normal(self, value, strict_level, expected):
        type_object = Dictionary(value, strict_level)

        assert type_object.is_type() == expected
        assert type_object.typecode == Typecode.DICTIONARY



"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

import decimal

from .._common import strip_ansi_escape
from .._const import DefaultValue, ParamKey
from ..error import TypeConversionError
from ._interface import AbstractValueConverter


class FloatConverter(AbstractValueConverter):
    def __init__(self, value, params):
        super().__init__(value, params)

        self.float_class = self._params.get("float_type")
        if self.float_class is None:
            self.float_class = DefaultValue.FLOAT_TYPE

    def force_convert(self):
        if isinstance(self._value, self.float_class):
            return self._value

        if isinstance(self._value, float):
            return self.float_class(str(self._value))

        try:
            return self.float_class(self._value)
        except (TypeError, ValueError, decimal.InvalidOperation):
            pass

        if self._params.get(ParamKey.STRIP_ANSI_ESCAPE, DefaultValue.STRIP_ANSI_ESCAPE):
            try:
                return self.float_class(strip_ansi_escape(self._value))
            except (TypeError, ValueError, decimal.InvalidOperation):
                pass

        raise TypeConversionError(
            "failed to force_convert to float: type={}".format(type(self._value))
        )



"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

import sys

import pytest

from typepy import is_empty_sequence, is_hex, is_not_empty_sequence


nan = float("nan")
inf = float("inf")


class Test_is_hex:
    @pytest.mark.parametrize(["value"], [["0x00"], ["0xffffffff"], ["a"], ["f"]])
    def test_normal(self, value):
        assert is_hex(value)

    @pytest.mark.parametrize(
        ["value"], [[None], [nan], [inf], [0], [1], [0.5], ["test"], ["g"], [True]]
    )
    def test_abnormal(self, value):
        assert not is_hex(value)


class Test_is_empty_sequence:
    @pytest.mark.parametrize(
        ["value", "expected"],
        [
            [(), True],
            [[], True],
            ["", True],
            [range(0), True],
            [[1], False],
            [["a"] * 20000, False],
            [(1,), False],
            [("a",) * 20000, False],
            ["aaa", False],
            [range(0, 10), False],
            [True, False],
            [False, False],
            [sys.maxsize, False],
            [0.1, False],
            [nan, False],
            [inf, False],
        ],
    )
    def test_normal(self, value, expected):
        assert is_empty_sequence(value) == expected


class Test_is_not_empty_sequence:
    @pytest.mark.parametrize(
        ["value", "expected"],
        [
            [[1], True],
            [["a"] * 20000, True],
            [(1,), True],
            [("a",) * 20000, True],
            ["a" * 20000, True],
            [range(0, 10), True],
            [(), False],
            [[], False],
            [None, False],
            [range(0), False],
            [True, False],
            [False, False],
            [sys.maxsize, False],
            [0.1, False],
            [nan, False],
            [inf, False],
        ],
    )
    def test_normal(self, value, expected):
        assert is_not_empty_sequence(value) == expected



"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

import sys
from datetime import date, datetime
from decimal import Decimal
from ipaddress import IPv4Address, IPv6Address

import pytest
from pytz import timezone, utc

import typepy
from typepy import StrictLevel


class Test_TypeClass_repr:
    @pytest.mark.parametrize(
        ["type_class", "value", "strict_level"],
        [
            [typepy.Integer, -sys.maxsize, StrictLevel.MIN],
            [typepy.Integer, sys.maxsize, StrictLevel.MAX],
            [typepy.RealNumber, -0.1, StrictLevel.MIN],
            [typepy.RealNumber, 0.1, StrictLevel.MAX],
            [typepy.DateTime, 1485685623, StrictLevel.MIN],
        ],
    )
    def test_smoke(self, type_class, value, strict_level):
        type_object = type_class(value, strict_level)

        assert str(type_object)


class Test_RealNumber:
    @pytest.mark.parametrize(
        ["value", "float_type", "expected"], [["0.1", float, 0.1], ["0.1", Decimal, Decimal("0.1")]]
    )
    def test_normal_float_type(self, value, float_type, expected):
        result = typepy.RealNumber(
            value, strict_level=StrictLevel.MIN, float_type=float_type
        ).convert()

        assert result == expected


class Test_DateTime:
    @pytest.mark.parametrize(
        ["value", "timezone", "expected"],
        [
            [datetime(2017, 1, 29, 10, 27, 3), utc, datetime(2017, 1, 29, 10, 27, 3)],
            [
                datetime(2017, 1, 29, 10, 27, 3),
                timezone("Asia/Tokyo"),
                datetime(2017, 1, 29, 10, 27, 3),
            ],
            ["2017-01-29 19:27:03", utc, datetime(2017, 1, 29, 19, 27, 3)],
            ["2017-01-29 19:27:03", timezone("Asia/Tokyo"), datetime(2017, 1, 29, 19, 27, 3)],
            [1485685623, utc, datetime(2017, 1, 29, 10, 27, 3)],
            [1485685623, timezone("Asia/Tokyo"), datetime(2017, 1, 29, 19, 27, 3)],
        ],
    )
    def test_normal_datetime_timezone(self, value, timezone, expected):
        result = typepy.DateTime(value, strict_level=StrictLevel.MIN, timezone=timezone).convert()

        assert result == timezone.localize(expected)

    @pytest.mark.parametrize(
        ["value", "expected"],
        [
            [date(2017, 1, 29), datetime(2017, 1, 29, 0, 0, 0)],
            ["2017-01-29", datetime(2017, 1, 29, 0, 0, 0)],
        ],
    )
    def test_normal_date(self, value, expected):
        result = typepy.DateTime(value, strict_level=StrictLevel.MIN).convert()

        assert result == expected


class Test_IpAddress:
    @pytest.mark.parametrize(
        ["value", "expected"],
        [
            [IPv4Address("127.0.0.1"), IPv4Address("127.0.0.1")],
            ["127.0.0.1", IPv4Address("127.0.0.1")],
            ["::1", IPv6Address("::1")],
        ],
    )
    def test_normal(self, value, expected):
        result = typepy.IpAddress(value, strict_level=StrictLevel.MIN).convert()

        assert result == expected



"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

from datetime import date, datetime

from ._checker import CheckerFactory, TypeCheckerBase, TypeCheckerDelegator
from ._common import isstring


class DateTimeTypeCheckerStrictLevel0(TypeCheckerBase):
    def is_instance(self):
        return isinstance(self._value, (date, datetime))


class DateTimeTypeCheckerStrictLevel1(DateTimeTypeCheckerStrictLevel0):
    def is_exclude_instance(self):
        from ..type._integer import Integer

        # exclude timestamp
        return Integer(self._value, strict_level=1).is_type()


class DateTimeTypeCheckerStrictLevel2(DateTimeTypeCheckerStrictLevel1):
    def is_exclude_instance(self):
        return isstring(self._value) or super().is_exclude_instance()


_factory = CheckerFactory(
    checker_mapping={
        0: DateTimeTypeCheckerStrictLevel0,
        1: DateTimeTypeCheckerStrictLevel1,
        2: DateTimeTypeCheckerStrictLevel2,
    }
)


class DateTimeTypeChecker(TypeCheckerDelegator):
    def __init__(self, value, strict_level):
        super().__init__(value=value, checker_factory=_factory, strict_level=strict_level)



"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

import decimal
import math


def isstring(value):
    return isinstance(value, (str,) + (str, bytes))


def isinf(value):
    try:
        return decimal.Decimal(value).is_infinite()
    except OverflowError:
        return True
    except TypeError:
        return False
    except (ValueError, decimal.InvalidOperation):
        return False


def isnan(value):
    try:
        return math.isnan(value)
    except (TypeError, OverflowError):
        return False



__author__ = "Tsuyoshi Hombashi"
__copyright__ = "Copyright 2017, {}".format(__author__)
__license__ = "MIT License"
__version__ = "1.1.4"
__maintainer__ = __author__
__email__ = "tsuyoshi.hombashi@gmail.com"



"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

import itertools
import sys
from decimal import Decimal

import pytest
from tcolorpy import tcolor

from typepy import Integer, StrictLevel, Typecode


class_under_test = Integer
nan = float("nan")
inf = float("inf")


class Test_Integer_is_type:
    @pytest.mark.parametrize(
        ["value", "strict_level", "expected"],
        [
            [str(sys.maxsize), StrictLevel.MIN, True],
            [str(sys.maxsize), StrictLevel.MIN + 1, True],
            [str(sys.maxsize), StrictLevel.MAX, False],
            ["45e76582", StrictLevel.MIN, True],
            ["45e76582", StrictLevel.MIN + 1, False],
            ["45e76582", StrictLevel.MAX, False],
            ["4.5e-4", StrictLevel.MIN, True],
            ["4.5e-4", StrictLevel.MIN + 1, False],
            ["4.5e-4", StrictLevel.MAX, False],
            [" 1 ", StrictLevel.MIN + 1, True],
            [True, StrictLevel.MIN + 1, False],
            [False, StrictLevel.MAX, False],
        ]
        + list(
            itertools.product(
                [0, sys.maxsize, -sys.maxsize, Decimal("1"), int(Decimal("45e765"))],
                [StrictLevel.MIN, StrictLevel.MIN + 1],
                [True],
            )
        )
        + list(
            itertools.product(
                [
                    0.5,
                    0.999,
                    Decimal("1.1"),
                    1e-05,
                    -1e-05,
                    "0.5",
                    ".999",
                    "1e-05",
                    "-1e-05",
                    True,
                    False,
                ],
                [StrictLevel.MIN],
                [True],
            )
        )
        + list(
            itertools.product(
                [None, nan, inf, "", "0xff", "test", "1a1", "11a", "a11", "テスト".encode()],
                [StrictLevel.MIN, StrictLevel.MAX],
                [False],
            )
        ),
    )
    def test_normal(self, value, strict_level, expected):
        type_checker = class_under_test(value, strict_level)

        assert type_checker.is_type() == expected
        assert type_checker.typecode == Typecode.INTEGER

    @pytest.mark.parametrize(
        ["value", "strip_ansi_escape", "expected"],
        [[tcolor("1", "red"), False, False], [tcolor("1", "red"), True, True]],
    )
    def test_normal_ansi(self, value, strip_ansi_escape, expected):
        type_checker = class_under_test(value, StrictLevel.MIN, strip_ansi_escape=strip_ansi_escape)

        assert type_checker.is_type() == expected
        assert type_checker.typecode == Typecode.INTEGER



"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

import itertools
from datetime import date, datetime

import pytest
from dateutil.tz import tzoffset
from tcolorpy import tcolor

from typepy import DateTime, StrictLevel, Typecode


dateutil = pytest.importorskip("dateutil", minversion="2.7")

class_under_test = DateTime
nan = float("nan")
inf = float("inf")


class Test_DateTime_is_type:
    @pytest.mark.parametrize(
        ["value", "strict_level", "expected"],
        list(
            itertools.product(
                [datetime(2017, 3, 22, 10, 0, tzinfo=tzoffset(None, 32400)), date(2017, 3, 22)],
                [StrictLevel.MIN, StrictLevel.MIN + 1, StrictLevel.MAX],
                [True],
            )
        )
        + list(
            itertools.product(
                [
                    None,
                    "invalid time string",
                    9223372036854775807,
                    "100-0004",
                    "テスト".encode(),
                ],
                [StrictLevel.MIN, StrictLevel.MIN + 1, StrictLevel.MAX],
                [False],
            )
        )
        + list(
            itertools.product(
                ["2017-03-22T10:00:00+0900", 1485685623, "1485685623", "1524930937.003555"],
                [StrictLevel.MIN],
                [True],
            )
        )
        + list(
            itertools.product(
                ["2017-03-22T10:00:00+0900", 1485685623, "1485685623"],
                [StrictLevel.MAX],
                [False],
            )
        ),
    )
    def test_normal(self, value, strict_level, expected):
        type_object = class_under_test(value, strict_level)

        assert type_object.is_type() == expected
        assert type_object.typecode == Typecode.DATETIME

    @pytest.mark.parametrize(
        ["value", "strip_ansi_escape", "expected"],
        [
            [tcolor("2017-03-22T10:00:00", "red"), False, False],
            [tcolor("2017-03-22T10:00:00", "red"), True, True],
        ],
    )
    def test_normal_ansi(self, value, strip_ansi_escape, expected):
        type_checker = class_under_test(value, StrictLevel.MIN, strip_ansi_escape=strip_ansi_escape)

        assert type_checker.is_type() == expected
        assert type_checker.typecode == Typecode.DATETIME



"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

import pytest

import typepy

from ._common import convert_wrapper


nan = float("nan")
inf = float("inf")


class Test_String:
    @pytest.mark.parametrize(
        ["method", "strict_level", "value", "expected"],
        [
            ["convert", 0, "abc", "abc"],
            ["convert", 0, "", ""],
            ["convert", 0, 1, "1"],
            ["convert", 0, "-1", "-1"],
            ["convert", 0, None, "None"],
            ["convert", 0, True, "True"],
            ["convert", 0, inf, "inf"],
            ["convert", 0, nan, "nan"],
            ["convert", 1, "abc", "abc"],
            ["convert", 1, "", ""],
            ["convert", 1, 1, "E"],
            ["convert", 1, "-1", "-1"],
            ["convert", 1, None, "E"],
            ["convert", 1, True, "E"],
            ["convert", 1, inf, "E"],
            ["convert", 1, nan, "E"],
            ["convert", 2, "abc", "abc"],
            ["convert", 2, "", "E"],
            ["convert", 2, 1, "E"],
            ["convert", 2, "-1", "-1"],
            ["convert", 2, None, "E"],
            ["convert", 2, True, "E"],
            ["convert", 2, inf, "E"],
            ["convert", 2, nan, "E"],
            ["try_convert", 0, "abc", "abc"],
            ["try_convert", 0, "", ""],
            ["try_convert", 0, 1, "1"],
            ["try_convert", 0, "-1", "-1"],
            ["try_convert", 0, None, "None"],
            ["try_convert", 0, True, "True"],
            ["try_convert", 0, inf, "inf"],
            ["try_convert", 0, nan, "nan"],
            ["try_convert", 1, "abc", "abc"],
            ["try_convert", 1, "", ""],
            ["try_convert", 1, 1, None],
            ["try_convert", 1, "-1", "-1"],
            ["try_convert", 1, None, None],
            ["try_convert", 1, True, None],
            ["try_convert", 1, inf, None],
            ["try_convert", 1, nan, None],
            ["try_convert", 2, "abc", "abc"],
            ["try_convert", 2, "", None],
            ["try_convert", 2, 1, None],
            ["try_convert", 2, "-1", "-1"],
            ["try_convert", 2, None, None],
            ["try_convert", 2, True, None],
            ["try_convert", 2, inf, None],
            ["try_convert", 2, nan, None],
            ["force_convert", 0, "abc", "abc"],
            ["force_convert", 0, "", ""],
            ["force_convert", 0, 1, "1"],
            ["force_convert", 0, "-1", "-1"],
            ["force_convert", 0, None, "None"],
            ["force_convert", 0, True, "True"],
            ["force_convert", 0, inf, "inf"],
            ["force_convert", 0, nan, "nan"],
            ["force_convert", 1, "abc", "abc"],
            ["force_convert", 1, "", ""],
            ["force_convert", 1, 1, "1"],
            ["force_convert", 1, "-1", "-1"],
            ["force_convert", 1, None, "None"],
            ["force_convert", 1, True, "True"],
            ["force_convert", 1, inf, "inf"],
            ["force_convert", 1, nan, "nan"],
            ["force_convert", 2, "abc", "abc"],
            ["force_convert", 2, "", ""],
            ["force_convert", 2, 1, "1"],
            ["force_convert", 2, "-1", "-1"],
            ["force_convert", 2, None, "None"],
            ["force_convert", 2, True, "True"],
            ["force_convert", 2, inf, "inf"],
            ["force_convert", 2, nan, "nan"],
        ],
    )
    def test_normal(self, method, strict_level, value, expected):
        assert convert_wrapper(typepy.String(value, strict_level), method) == expected


class Test_NullString:
    @pytest.mark.parametrize(
        ["method", "strict_level", "value", "expected"],
        [
            ["convert", 0, "", ""],
            ["convert", 0, " ", " "],
            ["convert", 1, "", ""],
            ["convert", 1, " ", " "],
            ["convert", 2, "", ""],
            ["convert", 2, " ", " "],
            ["try_convert", 0, "", ""],
            ["try_convert", 0, " ", " "],
            ["try_convert", 1, "", ""],
            ["try_convert", 1, " ", " "],
            ["try_convert", 2, "", ""],
            ["try_convert", 2, " ", " "],
            ["force_convert", 0, "", ""],
            ["force_convert", 0, " ", " "],
            ["force_convert", 1, "", ""],
            ["force_convert", 1, " ", " "],
            ["force_convert", 2, "", ""],
            ["force_convert", 2, " ", " "],
        ],
    )
    def test_normal(self, method, strict_level, value, expected):
        assert convert_wrapper(typepy.NullString(value, strict_level), method) == expected



"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

from typing import Any

from .._typecode import Typecode
from ._base import AbstractType


class Bool(AbstractType):
    """
    |result_matrix_desc|

    .. include:: matrix_bool_type.txt

    .. py:attribute:: strict_level

        |strict_level|
    """

    @property
    def typecode(self) -> Typecode:
        return Typecode.BOOL

    def __init__(self, value: Any, strict_level: int = 2, **kwargs) -> None:
        super().__init__(value, strict_level, **kwargs)

    def _create_type_checker(self):
        from ..checker._bool import BoolTypeChecker

        return BoolTypeChecker(self._data, self._strict_level)

    def _create_type_converter(self):
        from ..converter._bool import BoolConverter

        return BoolConverter(self._data, self._params)



"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

import itertools

import pytest

from typepy import is_not_null_string, is_null_string


nan = float("nan")
inf = float("inf")


class Test_is_not_null_string:
    @pytest.mark.parametrize(
        ["value", "expected"],
        list(itertools.product(["nan", "テスト"], [True]))
        + list(itertools.product([None, "", "  ", "\t", "\r\n", "\n", [], 1, True, nan], [False])),
    )
    def test_normal(self, value, expected):
        assert is_not_null_string(value) == expected


class Test_is_null_string:
    @pytest.mark.parametrize(
        ["value", "expected"],
        list(itertools.product([None, "", "  ", "\t", "\r\n", "\n"], [True]))
        + list(itertools.product(["nan", "テスト", [], 1, True, nan], [False])),
    )
    def test_normal(self, value, expected):
        assert is_null_string(value) == expected



"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

import itertools
import sys

import pytest

from typepy import NoneType, StrictLevel, Typecode


nan = float("nan")
inf = float("inf")


class Test_NoneTypeChecker_is_type:
    @pytest.mark.parametrize(
        ["value", "strict_level", "expected"],
        list(itertools.product([None], [StrictLevel.MIN, StrictLevel.MAX], [True]))
        + list(
            itertools.product(
                ["None", True, False, 0, sys.maxsize, inf, nan],
                [StrictLevel.MIN, StrictLevel.MAX],
                [False],
            )
        ),
    )
    def test_normal(self, value, strict_level, expected):
        expected_typecode = Typecode.NONE

        typeobj = NoneType(value, strict_level)

        assert typeobj.is_type() == expected
        assert typeobj.typecode == expected_typecode



"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

from ._checker import CheckerFactory, TypeCheckerBase, TypeCheckerDelegator


class DictionaryTypeCheckerStrictLevel0(TypeCheckerBase):
    def is_instance(self):
        return isinstance(self._value, dict)

    def is_valid_after_convert(self, converted_value):
        return isinstance(converted_value, dict) and converted_value


class DictionaryTypeCheckerStrictLevel1(DictionaryTypeCheckerStrictLevel0):
    def is_exclude_instance(self):
        return not isinstance(self._value, dict)


_factory = CheckerFactory(
    checker_mapping={0: DictionaryTypeCheckerStrictLevel0, 1: DictionaryTypeCheckerStrictLevel1}
)


class DictionaryTypeChecker(TypeCheckerDelegator):
    def __init__(self, value, strict_level):
        super().__init__(value=value, checker_factory=_factory, strict_level=strict_level)



"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

import itertools
import sys

import pytest

from typepy import NullString, StrictLevel, String, Typecode


nan = float("nan")
inf = float("inf")


class Test_String_is_type:
    @pytest.mark.parametrize(
        ["value", "strict_level", "expected"],
        list(itertools.product([], [StrictLevel.MIN, StrictLevel.MAX], [False]))
        + list(
            itertools.product(
                ["None", "いろは", "いろは".encode()], [StrictLevel.MIN, StrictLevel.MAX], [True]
            )
        )
        + list(
            itertools.product(
                ["", " ", "\n", sys.maxsize, inf, nan, None], [StrictLevel.MIN], [True]
            )
        )
        + list(
            itertools.product(
                ["", " ", "\n", sys.maxsize, inf, nan, None], [StrictLevel.MAX], [False]
            )
        ),
    )
    def test_normal(self, value, strict_level, expected):
        type_checker = String(value, strict_level=strict_level)

        assert type_checker.is_type() == expected
        assert type_checker.typecode == Typecode.STRING


class Test_NullString_is_type:
    @pytest.mark.parametrize(
        ["value", "strict_level", "expected"],
        [[None, StrictLevel.MIN, True]]
        + list(
            itertools.product(
                ["", " ", "\t", "\n", " \r\n"], [StrictLevel.MIN, StrictLevel.MAX], [True]
            )
        )
        + list(
            itertools.product(
                [sys.maxsize, "None", inf, "いろは", "いろは".encode()],
                [StrictLevel.MIN, StrictLevel.MAX],
                [False],
            )
        ),
    )
    def test_normal(self, value, strict_level, expected):
        type_object = NullString(value, strict_level)

        assert type_object.is_type() == expected
        assert type_object.typecode == Typecode.NULL_STRING



"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

from ..error import TypeConversionError
from ._interface import AbstractValueConverter


class ListConverter(AbstractValueConverter):
    def force_convert(self):
        try:
            return list(self._value)
        except (TypeError, ValueError):
            raise TypeConversionError(
                "failed to force_convert to list: type={}".format(type(self._value))
            )



"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

import itertools
import sys
from decimal import Decimal

import pytest
from tcolorpy import tcolor

from typepy import RealNumber, StrictLevel, Typecode


class_under_test = RealNumber
nan = float("nan")
inf = float("inf")


class Test_RealNumber_is_type:
    @pytest.mark.parametrize(
        ["value", "strict_level", "expected"],
        list(
            itertools.product(
                [
                    0,
                    0.0,
                    0.0,
                    sys.maxsize,
                    -sys.maxsize,
                    str(sys.maxsize),
                    str(-sys.maxsize),
                    "0.1",
                    "-0.1",
                    "1e-05",
                    "4.5e-4",
                    "45e76",
                    int(Decimal("45e765")),
                ],
                [StrictLevel.MIN],
                [True],
            )
        )
        + list(
            itertools.product(
                [0, 0.0, 0.0, sys.maxsize, -sys.maxsize, "4.5e444"], [StrictLevel.MIN + 1], [False]
            )
        )
        + list(itertools.product(["1.1", "4.5e-4"], [StrictLevel.MIN + 1], [True]))
        + list(
            itertools.product(
                [
                    0,
                    0.0,
                    Decimal("1"),
                    sys.maxsize,
                    -sys.maxsize,
                    "1.0",
                    str(sys.maxsize),
                    str(-sys.maxsize),
                    "0.1",
                    "-0.1",
                    "1e-05",
                ],
                [StrictLevel.MAX],
                [False],
            )
        )
        + list(
            itertools.product(
                [True, inf, nan, "", "0xf", "テスト".encode()],
                [StrictLevel.MIN, StrictLevel.MIN + 1, StrictLevel.MAX],
                [False],
            )
        )
        + list(
            itertools.product(
                [0.1, -0.1, 0.5, Decimal("1.1")],
                [StrictLevel.MIN, StrictLevel.MIN + 1, StrictLevel.MAX],
                [True],
            )
        ),
    )
    def test_normal(self, value, strict_level, expected):
        type_checker = class_under_test(value, strict_level)

        assert type_checker.is_type() == expected
        assert type_checker.typecode == Typecode.REAL_NUMBER

    @pytest.mark.parametrize(
        ["value", "strip_ansi_escape", "expected"],
        [[tcolor("1.1", "red"), False, False], [tcolor("1.1", "red"), True, True]],
    )
    def test_normal_ansi(self, value, strip_ansi_escape, expected):
        type_checker = class_under_test(value, StrictLevel.MIN, strip_ansi_escape=strip_ansi_escape)

        assert type_checker.is_type() == expected
        assert type_checker.typecode == Typecode.REAL_NUMBER



"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

import os.path
from typing import Dict

import setuptools


MODULE_NAME = "typepy"
REPOSITORY_URL = "https://github.com/thombashi/{:s}".format(MODULE_NAME)
REQUIREMENT_DIR = "requirements"
ENCODING = "utf8"

pkg_info = {}  # type: Dict[str, str]


def get_release_command_class() -> Dict[str, setuptools.Command]:
    try:
        from releasecmd import ReleaseCommand
    except ImportError:
        return {}

    return {"release": ReleaseCommand}


with open(os.path.join(MODULE_NAME, "__version__.py")) as f:
    exec(f.read(), pkg_info)

with open("README.rst", encoding=ENCODING) as f:
    long_description = f.read()

with open(os.path.join("docs", "pages", "introduction", "summary.txt"), encoding=ENCODING) as f:
    summary = f.read().strip()

with open(os.path.join(REQUIREMENT_DIR, "requirements.txt")) as f:
    install_requires = [line.strip() for line in f if line.strip()]

with open(os.path.join(REQUIREMENT_DIR, "test_requirements.txt")) as f:
    tests_requires = [line.strip() for line in f if line.strip()]


DATETIME_REQUIRES = [
    "python-dateutil>=2.8.0,<3.0.0",
    "pytz>=2018.9",
]

setuptools.setup(
    name=MODULE_NAME,
    version=pkg_info["__version__"],
    url=REPOSITORY_URL,
    author=pkg_info["__author__"],
    author_email=pkg_info["__email__"],
    description=summary,
    include_package_data=True,
    keywords=["library", "type-checking", "type-conversion", "validator"],
    license=pkg_info["__license__"],
    long_description=long_description,
    long_description_content_type="text/x-rst",
    packages=setuptools.find_packages(exclude=["test*"]),
    package_data={MODULE_NAME: ["py.typed"]},
    project_urls={
        "Documentation": "https://{:s}.rtfd.io/".format(MODULE_NAME),
        "Source": REPOSITORY_URL,
        "Tracker": "{:s}/issues".format(REPOSITORY_URL),
    },
    python_requires=">=3.5",
    install_requires=install_requires,
    extras_require={"datetime": DATETIME_REQUIRES, "test": tests_requires + DATETIME_REQUIRES},
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Intended Audience :: Information Technology",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Topic :: Software Development :: Libraries",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    cmdclass=get_release_command_class(),
)



"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

import itertools
import sys
from ipaddress import ip_address

import pytest
from tcolorpy import tcolor

from typepy import IpAddress, StrictLevel, Typecode


class_under_test = IpAddress
nan = float("nan")
inf = float("inf")


class Test_IpAddress_is_type:
    @pytest.mark.parametrize(
        ["value", "strict_level", "expected"],
        [[tcolor("127.0.0.1", "red"), StrictLevel.MIN, True]]
        + list(
            itertools.product(
                ["", " ", sys.maxsize, str(sys.maxsize), inf, nan, None],
                [StrictLevel.MIN, StrictLevel.MAX],
                [False],
            )
        )
        + list(
            itertools.product(
                [ip_address("127.0.0.1"), ip_address("::1")],
                [StrictLevel.MIN, StrictLevel.MAX],
                [True],
            )
        )
        + list(itertools.product(["127.0.0.1", "::1", "800::800"], [StrictLevel.MIN], [True]))
        + list(itertools.product(["127.0.0.1", "::1", "800::800"], [StrictLevel.MAX], [False])),
    )
    def test_normal(self, value, strict_level, expected):
        type_checker = IpAddress(value, strict_level=strict_level)

        assert type_checker.is_type() == expected
        assert type_checker.typecode == Typecode.IP_ADDRESS

    @pytest.mark.parametrize(
        ["value", "strip_ansi_escape", "expected"],
        [[tcolor("127.0.0.1", "red"), False, False], [tcolor("127.0.0.1", "red"), True, True]],
    )
    def test_normal_ansi(self, value, strip_ansi_escape, expected):
        type_checker = class_under_test(value, StrictLevel.MIN, strip_ansi_escape=strip_ansi_escape)

        assert type_checker.is_type() == expected
        assert type_checker.typecode == Typecode.IP_ADDRESS



"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

import typepy.type

from .__version__ import __author__, __copyright__, __email__, __license__, __version__
from ._const import ParamKey, StrictLevel
from ._function import (
    extract_typepy_from_dtype,
    is_empty_sequence,
    is_hex,
    is_not_empty_sequence,
    is_not_null_string,
    is_null_string,
)
from ._typecode import Typecode
from .error import TypeConversionError
from .type import (
    Binary,
    Bool,
    Bytes,
    DateTime,
    Dictionary,
    Infinity,
    Integer,
    IpAddress,
    List,
    Nan,
    NoneType,
    NullString,
    RealNumber,
    String,
)



"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

import typepy


EXCEPTION_RESULT = "E"


def convert_wrapper(typeobj, method):
    try:
        return getattr(typeobj, method)()
    except (typepy.TypeConversionError):
        return EXCEPTION_RESULT
